// Utils
try {
	$ // Test if it is already used
} catch(e) {
	$ = function(id) {
		return document.getElementById(id)
	};
}

var gMMBridge_SendEventBatchTimer;
var gMMBridge_QueuedTrackingEvents;
var gMMBridge_SavedUnloadHandler;
var gMMBridge_sendTrackingEventsTimer; //ms

// Acceleration Handling
var gSendTimestamps = false;

// A little more abstract
var DEBUG = true;
if (!window.console || !DEBUG) {
	console = {
		log: function(){ },
		error: function(){ }
	}
}

function queueTrackingEvent(eventType, eventKey, eventVal)
{
	if (eventType)
	{
		var ev = escape(eventType) +
		(eventKey || eventVal ? "?" : "") + 
		(eventKey ? "key="+escape(eventKey) : "") + 
		(eventVal ? ((eventKey ? "&" : "" ) +  "val="+escape(eventVal)) : "");

		if (!gMMBridge_QueuedTrackingEvents)
		{
			gMMBridge_QueuedTrackingEvents = new Array();
		}
		gMMBridge_QueuedTrackingEvents[gMMBridge_QueuedTrackingEvents.length]=ev;		
	
		if (!gMMBridge_sendTrackingEventsTimer)
			gMMBridge_sendTrackingEventsTimer = setTimeout(sendQueuedTrackingEvents, 100); // every 100 ms, flush events
	}
}


// will return all queued events, serialized in the order they were received
// each event will look like: type=<TYPE>&key=<KEY>&val=<DATA>
function serializeTrackingEvents()
{
	
	var retStr;
	for (var i in gMMBridge_QueuedTrackingEvents) {
		var str = escape(gMMBridge_QueuedTrackingEvents[i]);
		retStr = retStr ? (retStr + "&" + str) : (str); 
	}
	return retStr;
}

function sendQueuedTrackingEvents()
{
	var events = serializeTrackingEvents();
	if (events)
	{
		call_MMBridgeFunc("batchedEvents?" + events);
	}
	gMMBridge_QueuedTrackingEvents = null;	
	clearTimeout(gMMBridge_sendTrackingEventsTimer);
	gMMBridge_sendTrackingEventsTimer=null;
}

function call_MMBridgeFunc(funcargs)
{
	document.location=("mmbridge:" + funcargs);

	//DEBUGGING ONLY	//DEBUG -remove
	//alert("mmbridge:" + funcargs);
}

function MMBridge_saveunload()
{
	gMMBridge_SavedUnloadHandler = window.onunload;	
	window.onunload = MMBridge_unload;
}

function MMBridge_unload()
{
	// flush any unqueued events
	if (gMMBridge_SavedUnloadHandler)
		gMMBridge_SavedUnloadHandler(); //important to call this first, it might send some events!

	if (gMMBridge_sendTrackingEventsTimer)
	{
		clearTimeout(gMMBridge_sendTrackingEventsTimer);
		sendQueuedTrackingEvents();
	}

	return true;
}

function _unsupported()
{
	alert("MMBridge/PhoneGap is not supported!")
}

var Device = {
	available: false,
	bridgeVersion: "",
	model: "",
	version: "",
	uuid: "",
	initCallback: null,
	init: function(bridgeVersion, model, version, uuid) {
		try {
			//save the document's onunload handler, so we can call it after our own
			MMBridge_saveunload();
			Device.available = true;
			Device.bridgeVersion = bridgeVersion;
			Device.model = model;
			Device.version = version;
			Device.uuid = uuid;
			if(Device.initCallback != null) {
				Device.initCallback();
				Device.initCallback = null;
			}
		} catch(e) {
			// alert("MMBridge is not supported!");
		}
	},
	launchYoutube: function(urlStr) {
		if (Device.available) {
			try {
				call_MMBridgeFunc("launchYoutube?url=" + escape(urlStr));
			} catch(e) {
				console.log("Error logging message '" + message + "' - " + e);
			}
		}
	},
	log: function(message) {
		if (Device.available) {
			try {
				call_MMBridgeFunc("log?message=" + escape(message));
			} catch(e) {
				console.log("Error logging message '" + message + "' - " + e);
			}
		}
	},
	exec: function(command) {
		if (Device.available) {
			try {
				call_MMBridgeFunc(command);
			} catch(e) {
				console.log("Command '" + command + "' has not been executed, because of exception: " + e);
				alert("Error executing command '" + command + "'.");
			}
		}
	},
	DeviceID: {
		callback: null,
		init: function(func) 
		{
			Device.DeviceID.callback = func;
		},
		set: function(deviceID)
		{
			if (Device.DeviceID.callback && Device.available)
			{
				Device.DeviceID.callback(deviceID);	
			}
		},
		get: function()
		{
		try 
		{
			call_MMBridgeFunc('getDeviceID');
		} 
			catch(e) 
		{
			_unsupported();
		}
		},
	},
	SDKVersion: {
		callback: null,
		init: function(func) 
		{
			Device.SDKVersion.callback = func;
		},
		set: function(sdkVersion)
		{
			if (Device.SDKVersion.callback && Device.available)
			{
				Device.SDKVersion.callback(sdkVersion);	
			}
		},
		get: function()
		{
		try 
		{
			call_MMBridgeFunc('getSDKVersion');
		} 
			catch(e) 
		{
			_unsupported();
		}
		},
	},
	BatteryLevel: {
		callback: null,
		init: function(func) 
		{
			Device.BatteryLevel.callback = func;
		},
		set: function(batteryLevel)
		{
			if (Device.BatteryLevel.callback && Device.available)
			{
				Device.BatteryLevel.callback(batteryLevel);	
			}
		},
		get: function()
		{
		try 
		{
			call_MMBridgeFunc('getBatteryLevel');
		}
		catch(e) 
		{
			_unsupported();
		}
		},
	},
	AdData: {
		onExpandStart: 
		{
		callback: null,
		init: function(func) 
		{
			Device.AdData.onExpandStart.callback = func;
		},
		set: function()
		{
			if (Device.AdData.onExpandStart.callback && Device.available)
			{
				Device.AdData.onExpandStart.callback();	
			  }
		},
	},
	onExpandComplete: 
	{ 
		callback: null,
		init: function(func) 
		{
			Device.AdData.onExpandComplete.callback = func;
		},
		set: function()
		{
			if (Device.AdData.onExpandComplete.callback && Device.available)
			{
				Device.AdData.onExpandComplete.callback();	
			}
		},
	},
	onCollapseStart: 
	{
		callback: null,
		init: function(func) 
		{
			Device.AdData.onCollapseStart.callback = func;
		},
		set: function()
		{
			if (Device.AdData.onCollapseStart.callback && Device.available)
			{
				Device.AdData.onCollapseStart.callback();	
			}
		},
	},
	onCollapseComplete: 
	{
		callback: null,
		init: function(func) 
		{
			Device.AdData.onCollapseComplete.callback = func;
		},
		set: function()
		{
			if (Device.AdData.onCollapseComplete.callback && Device.available)
			{
				Device.AdData.onCollapseComplete.callback();	
			}
		},
	},
	adID: 
	{
		callback: null,
		init: function(func) 
		{
			Device.AdData.adID.callback = func;
		},
		set: function(adID)
		{
			if (Device.AdData.adID.callback && Device.available)
			{
				Device.AdData.adID.callback(adID);	
			}
		},
		get: function()
		{
			try 
			{
				call_MMBridgeFunc('getAdID');
			} 
			catch(e) 
			{
				_unsupported();
			}
		},
	},
},
	Location: 
	{
		callback: null,		
		init: function(func) 
		{
			Device.Location.callback = func;
		},
		set: function(lat, lon,locationAvailable) 
		{
			if (Device.Location.callback && Device.available)
			{
				Device.Location.callback(lat,lon,locationAvailable);	
			}
		},
		get: function()
		{
		try 
		{
			call_MMBridgeFunc('getloc');
		} 
		catch(e) 
		{
			_unsupported();
		}
		},
	},
	vibrate: function() {
		return Device.exec("vibrate");
	},
	sendTimestamps: function(bSend) {
		if (bSend && typeof bSend == "boolean")
		{
			gSendTimestamps = bSend;
		} else {
			gSendTimestamps = true;
		}
	},
	Orientation: {
		angle: 0,
		callback: null,
		init: function(func) 
		{
			Device.Orientation.callback = func;
			try {
				call_MMBridgeFunc('initOrientation');
			} catch(e) {
				_unsupported();
			}
		},
		set: function(position)
		{
			if (Device.Orientation.callback && Device.available)
			{
				Device.Orientation.callback(position);	
			}
		},
		get: function() {
			try {
				call_MMBridgeFunc('initOrientation');
			} catch(e) {
				_unsupported();
			}
		},
	},
	Acceleration: {
		accelX: 0,
		accelY: 0,
		accelZ: 0,
		rollingX: 0,
		rollingY: 0,
		rollingZ: 0,		
		updateFreq: 1.0/40.0,
		shakeThreshold: 2.0, 
		filteringFactor: 0.1, 
		callback: null,
		shakeCallback: null,
		_callback:function(x, y, z, rx, ry, rz)
		{
			if (Device.Acceleration.callback && Device.available)
			{
				Device.Acceleration.callback(x, y, z, rx, ry, rz);	
			}
		},
		observe:function(bStart, optFreq) {
			try {
				if (!(typeof bStart == "boolean"))
					bStart = true;
				if (optFreq && !isNaN(parseFloat(optFreq)))
				{	
					Device.Acceleration.updateFreq = (optFreq == 0.0 ) ? 1.0/40.0 : optFreq;
				}
				call_MMBridgeFunc('getaccel?start=' + bStart +
									  '&freq='+Device.Acceleration.updateFreq
									+ '&filteringFactor=' + Device.Acceleration.filteringFactor
									+ '&shakeThreshold=' + Device.Acceleration.shakeThreshold
									);
			} catch(e) {
				_unsupported();
			}
		}		
	},
	
	observeAccel: function(bStart, freq) { /* can go away in favor of Document.Acceleration.observe(bStart, freq)*/
		Document.Acceleration.observe(bStart, freq);
	},
	playsound: function(soundname) {
		try {
				if (!Device || !Device.available)
				{
					setTimeout("Device.playsound(" + soundname + ")", 500);
				} else {	
					call_MMBridgeFunc('playsound?file=' + escape(soundname));
				}
		} catch(e) {
			_unsupported();;
		}
	},
	trackingEvent : {
		setString: function(strKey, strValue) {
			try {
					queueTrackingEvent("setTrackingEventString", strKey, strValue);
				} catch(e) {
					_unsupported();
				}
			},
		setDurationForKey: function(strKey, duration) {
			try {
					queueTrackingEvent("setDurationForKey", strKey, parseFloat(duration));
				} catch(e) {
					_unsupported();
				}
			},
		startTimerForKey: function(strKey) {
			try {
					queueTrackingEvent("startTimerForKey", strKey);
				} catch(e) {
					_unsupported();
				}
			},
		endTimerForKey: function(strKey) {
			try {
					queueTrackingEvent("endTimerForKey", strKey);
				} catch(e) {
					_unsupported();
				}
			},
		incrementKey: function(strKey, optDurationMs) {
			try {
				queueTrackingEvent("incrementKey", escape(strKey), parseFloat(_mm_timeStampString()));
				//call_MMBridgeFunc('incrementKey?key=' + escape(key),_mm_timeStampString())
			} catch(e) {
				_unsupported();
			}
		},
		setNumber: function(strKey, numValue) {
			try {
					queueTrackingEvent("setTrackingEventNumber", escape(strKey), parseFloat(numValue));
				} catch(e) {
					_unsupported();
				}
			},
		setValue: function(strKey, inval) {
			try {
					if (typeof inval == "string")
					{
						queueTrackingEvent("setTrackingEventString", escape(strKey), inval);					
					}
					else if (typeof inval == "number")
					{
						queueTrackingEvent("setTrackingEventNumber", escape(strKey), parseFloat(inval));
					}
				} catch(e) {
					_unsupported();
				}
			},
		setValues: function(dataobj) {
			try {
					for (var i in dataobj) {
						if (i && typeof i == "string") {
							if (dataobj[i] && typeof(dataobj[i]) == "string")
							{
								queueTrackingEvent("setTrackingEventString", i, dataobj[i]);
							}
							else if (dataobj[i] && typeof(dataobj[i]) == "number")
							{
								queueTrackingEvent("setTrackingEventNumber", i, parseFloat(dataobj[i]));
							}
						}
					}
				} catch(e) {
					_unsupported();
				}
			}
			
	},
	sendData: function(dataobj, optDuration) {
		try {
			var serialdata = "";
			for (var i in dataobj) {
				if (i && typeof i == "string") {
					var val = "";
					if (dataobj[i])
					{
						val = escape(dataobj[i]);
					}
					if (serialdata == "")
						serialdata += "?" + escape(i) + "=" + val;
					else
						serialdata += "&" + escape(i) + "=" + val;
				}
			}
			if (optDuration && optDuration != null && !isNaN(optDuration))
			{
				serialdata += "&MMUserDuration=" + optDuration.toString();
			}
			call_MMBridgeFunc('sendData' + serialdata + _mm_timeStampString());
		} catch(e) {
			_unsupported();
		}
	},
	dismissAd: function() {
		try {
			call_MMBridgeFunc('dismissAd');
		} catch(e) {
			_unsupported();
		}
	},
	launchApp: function(appURL,alternateID) {
		try {
			call_MMBridgeFunc('launchApp?appURL=' + escape(appURL) + "&alternateID=" + escape(alternateID));
		} catch(e) {
			_unsupported();
		}
	},
	playVideo: function(videoURL,videoKey) {
		try {
			var videoRequest = 'playVideo?videoURL=' + escape(videoURL);
			if (videoKey && videoKey != null)
			{
				videoRequest += "&videoKey=" + escape(videoKey);
			}
			call_MMBridgeFunc(videoRequest);
		} catch(e) {
			_unsupported();
		}
	},
	expandAd: function() {
		try {
			call_MMBridgeFunc('expandAd');
		} catch(e) {
			_unsupported();
		}
	}
}


function _mm_timeStampString()
{
	if (gSendTimestamps)
	{
		return "&_MM_Timestamp=" + (Math.floor(new Date().getTime()/1000)).toString(); //Fixme - convert to a ISO-8601, or some other standard?
	}
	else
		return "";
}

//wrappers for user convenience
var MMBridge = Device;
